#! /usr/bin/env fansubstitute
//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   6 Nov 06  Brian Frank  Creation
//

using build

**
** buildall.fan
**
** This is the top level build script for building Fan - it routes
** build functions to two sub-scripts buildboot and buildpods.  This
** top level script itself and buildboot.fan must be hosted by a
** substitute runtime defined in sys.props used to bootstrap this
** development environment to a self-buildable state.
**
class Build : BuildGroup
{

//////////////////////////////////////////////////////////////////////////
// Setup
//////////////////////////////////////////////////////////////////////////

  override Void setup()
  {
    childrenScripts =
    [
      `buildboot.fan`,
      `buildpods.fan`,
    ]
  }

  override Target[] makeTargets()
  {
    return BuildScript.super.makeTargets
  }

//////////////////////////////////////////////////////////////////////////
// Compile
//////////////////////////////////////////////////////////////////////////

  @target="run compile on all pods"
  Void compile()
  {
    spawnOnChildren("compile")
  }

//////////////////////////////////////////////////////////////////////////
// Clean
//////////////////////////////////////////////////////////////////////////

  @target="run clean on all pods"
  Void clean()
  {
    runOnChildren("clean")
  }

//////////////////////////////////////////////////////////////////////////
// Full
//////////////////////////////////////////////////////////////////////////

  @target="run full on all pods"
  Void full()
  {
    runOnChildren("clean")
    spawnOnChildren("full")
  }

//////////////////////////////////////////////////////////////////////////
// Test
//////////////////////////////////////////////////////////////////////////

  @target="run test on all pods"
  Void test()
  {
    fantExe := (binDir+"fant$exeExt".toUri).osPath
    Exec.make(this, [fantExe, "-all"]).run
  }

//////////////////////////////////////////////////////////////////////////
// Doc
//////////////////////////////////////////////////////////////////////////

  @target="build fandoc HTML docs"
  Void doc()
  {
    fanExe := (binDir+"fan$exeExt".toUri).osPath
    allBuildPodScripts.each |BuildPod script|
    {
      if (script.podName.startsWith("test")) return
      Exec.make(this, [fanExe, "docCompiler", script.podName]).run
    }

    Exec.make(this, [fanExe, "docCompiler", "-topindex"]).run
  }

//////////////////////////////////////////////////////////////////////////
// Superclean
//////////////////////////////////////////////////////////////////////////

  @target="delete lib dir"
  Void superclean()
  {
    Delete.make(this, libFanDir).run
    Delete.make(this, libJavaDir).run
    deleteNonDist
  }

//////////////////////////////////////////////////////////////////////////
// Dist
//////////////////////////////////////////////////////////////////////////

  @target="build distributation full, test, doc"
  Void dist()
  {
    superclean
    full
    doc
    test
    deleteNonDist
  }

  @target="delete non-distribuation files"
  Void deleteNonDist()
  {
    Delete.make(this, devHomeDir+`tmp/`).run
    Delete.make(this, devHomeDir+`lib/tmp/`).run
    Delete.make(this, devHomeDir+`lib/types.db`).run
    Delete.make(this, devHomeDir+`lib/tmp`).run
    Delete.make(this, devHomeDir+`src/jfan/temp/`).run

    libJavaDir.list.each |File f|
    {
      if (f.name != "sys.jar")
        Delete.make(this, f).run
    }
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  BuildPod[] allBuildPodScripts(BuildScript[] children := this.children)
  {
    // recursively find all the BuildPod scripts in
    // my children scripts and their descendents
    acc := BuildPod[,]
    children.each |BuildScript child|
    {
      if (child is BuildGroup)
      {
        group := child as BuildGroup
        acc.addAll((BuildPod[])group.children.findType(BuildPod.type))
        acc.addAll(allBuildPodScripts(group.children))
      }
    }
    return acc
  }

}
