//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   24 May 07  Andy Frank  Creation
//

using fandoc

**
** DocCompilerTest
**
class DocCompilerTest : Test
{

//////////////////////////////////////////////////////////////////////////
// TypeToDisplayName
//////////////////////////////////////////////////////////////////////////

  Void testTypeToDisplayName()
  {
    verifyName(Int.type, "Int")
    verifyName(Duration[,].type, "Duration[]")
    verifyName(Int:Str.type, "Int:Str")
    verifyName(Int:Str[,].type, "Int:Str[]")
    verifyName(|Bool a|.type, "|Bool|")
    verifyName(|Bool a, Str b|.type, "|Bool, Str|")
    verifyName(|Bool a, Str b -> Int|.type, "|Bool, Str -> Int|")
    verifyName(|Bool a, Str[] b -> Int|.type, "|Bool, Str[] -> Int|")
    verifyName(|Int:Str a -> Bool|.type, "|Int:Str -> Bool|")
  }

  Void verifyName(Type t, Str sig)
  {
    test := ApiToHtmlGenerator.typeToDisplay(t)
    verifyEq(test, sig, "$test != $sig   $t.signature")
  }

//////////////////////////////////////////////////////////////////////////
// FirstSentanceTest
//////////////////////////////////////////////////////////////////////////

  Void testFirstSentanceTest()
  {
    verifySentence("Foo bar", "Foo bar")
    verifySentence("Foo bar.", "Foo bar.")
    verifySentence("Foo bar\n", "Foo bar")
    verifySentence("Foo bar\n ", "Foo bar")
    verifySentence("Foo bar\n\n", "Foo bar")
    verifySentence("Foo bar\n\n ", "Foo bar")
    verifySentence("Foo bar. Alpha beta", "Foo bar.")
    verifySentence("Foo bar.\nAlpha beta", "Foo bar.")
    verifySentence("Foo bar.\n\nAlpha beta", "Foo bar.")
    verifySentence("Foo bar\nAlpha beta", "Foo bar Alpha beta")
    verifySentence("Foo\nBar\nAlpha\nBeta", "Foo Bar Alpha Beta")
    verifySentence("Foo\nBar\nAlpha\nBeta.", "Foo Bar Alpha Beta.")
    verifySentence("Foo<bar> alpha & beta.", "Foo&lt;bar&gt; alpha &amp; beta.")
  }

  Void verifySentence(Str para, Str sentence)
  {
    verifyEq(PodIndexGenerator.firstSentence(para), sentence)
  }

}
