//
// Copyright (c) 2008, John Sublett
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Jan 08  John Sublett  Creation
//

using sql

**
** ColDef for the Str data type.
**
@havenCol=Str.type
const internal class StrCol : ColDef
{
  new make() : super.make() {}

  new makeForField(HavenService haven, Field field) : super(haven, field)
  {
    len = field.facet("max", null)
    if (len == null)
      throw HavenErr.make("'max' facet required for Str fields.");
    isClob := field.facet("clob", false)
    if (!isClob)
      sqlTypeDef = "VARCHAR($len)"
    else
      sqlTypeDef = haven.dialect.getClobType(len)
  }
  
  override Str toSql(Obj o)
  {
    return "'" + safe((Str)o) + "'"
  }
  
  private Str safe(Str s)
  {
    return s.split("'").join("''")
  }
  
  override Void setFieldFromRow(Obj obj, Row row, HavenNamespace ns)
  {
    field.set(obj, row.get((Col)row.type.field(field.name)))
  }
  
  override Obj rowValue(Row row, Col col)
  {
    return row.get(col)
  }
  
  override Str sqlType()
  {
    return sqlTypeDef
  }
  
  const Int len
  const Str sqlTypeDef
}