//
// Copyright (c) 2008, John Sublett
// Licensed under the Academic Free License version 3.0
//
// History:
//   01 Jan 08  John Sublett  Creation
//

@serializable internal class StrNoMax
{
  Str s
}

@serializable internal class NotNullNoDefault
{
  Int x
}

@serializable internal class AutoStr
{
  @auto Str s
}

internal class NotSerializable
{
}

@serializable internal class TestRecord
{
  @key Int i := 57
  
  Float f := 3.14
  
  Bool b := true
  
  @max=16
  Str s := ""
  
  DateTime d := DateTime.makeTicks(0)
  
  MeddlingKid k := MeddlingKid.shaggy
  
  TestSimple t := TestSimple.make("zoinks")
}

@serializable internal class Test2
{
  @auto @key Int id := 0
  
  @max=64
  Str s := ""
}

@serializable internal class BigBuf
{
  @auto @key Int i := -1

  @max=50000
  Buf buf := Buf.make(32)
}

@serializable internal class BigClob
{
  @auto @key Int i := -1

  @max=50000 @clob
  Str clob := "clobberin time"
}

@simple @max=32 internal class TestSimple
{
  new make(Str foo) { this.foo = foo }
  override Str toStr() { return foo }
  static Obj fromStr(Str s) { return TestSimple.make(s) }
  override Bool equals(Obj that)
  {
    return (that.type == TestSimple.type) && foo.equals(((TestSimple)that).foo)
  }
  override Int hash() { return foo.hash }
  
  private Str foo
}

internal enum MeddlingKid { freddy, daphne, velma, shaggy }

@serializable internal class AutoInc
{
  @auto @key Int x := 0
}

@serializable internal class Fish
{
  new make(Str name := "", Float weight := 5.0)
  {
    this.name = name
    this.weight = weight
  }
  
  @key @max=16 Str name := ""
  Float weight := 5.0
  
  @allowNull @ref=Pond.type
  Uri pond
}

@serializable internal class Pond
{
  new make(Str name := "")
  {
    this.name = name
  }

  @auto @key Int id := -1
  @max=16 Str name := ""
}

@serializable internal class Indexed
{
  @index Int a := 0
  
  @index="bc"
  Int b := 0
  
  @index="bc"
  @max=30 Str c := ""
  
  @index="desc"
  Int d := 0
  
  @index="ef desc"
  Int e := 0
  
  @index="ef desc"
  Int f := 0
}

//////////////////////////////////////////////////////////////////////////
// Library types
//////////////////////////////////////////////////////////////////////////

**
** Book.
**
@serializable internal class Book
{
  @auto @key
  Int id := 0
  
  @max=64
  Str isbn := ""
  
  @max=64
  Str title := ""
  
  @ref=Person.type
  Uri author
  
  @ref=Publisher.type
  Uri publisher
}

**
** Chapter.
**
@serializable internal class Chapter
{
  @ref=Book.type
  Uri book
  
  Int number := 1
  
  @max=32
  Str title := ""
}

**
** A person who can author books or use a library.
**
@serializable internal class Person
{
  new make() {}

  new makeNamed(Str name)
  {
    this.name = name
  }
  
  @auto @key
  Int id := 0

  @max=32
  Str name := ""
}

**
** Publisher of a book.
**
@serializable internal class Publisher
{
  @auto @key
  Int id := 1
  
  @max=32
  Str name := ""
}