//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   25 Aug 06  Brian Frank  Creation
//

**
** ExprTest
**
class ExprTest : CompilerTest
{

//////////////////////////////////////////////////////////////////////////
// Literals
//////////////////////////////////////////////////////////////////////////

  Void testLiterals()
  {
    // null
    verifyExpr("null", null)

    // bool
    verifyExpr("false", false)
    verifyExpr("true", true)

    // int
    verifyExpr("0", 0)
    verifyExpr("1", 1)
    verifyExpr("0xabcd_1234_fedc_9876", 0xabcd_1234_fedc_9876)
    verifyExpr("493859850", 493859850)
    verifyExpr("-1", -1)
    verifyExpr("-123_456", -123_456)

    // float
    verifyExpr("0f",  0.0)
    verifyExpr("0.0", 0.0)
    verifyExpr("1.0", 1.0)
    verifyExpr("1.2", 1.2)
    verifyExpr("-1.2", -1.2)

    // str
    verifyExpr("\"\"",  "")
    verifyExpr("\"x\"", "x")
    verifyExpr("\"x\\ny\"", "x\ny")

    // duration
    verifyExpr("0ns",     0ns)
    verifyExpr("1ms",     1000_000ns)
    verifyExpr("1.2sec",  1_200_000_000ns)
    verifyExpr("1.5min",  90_000_000_000ns)
    verifyExpr("1hr",     3_600_000_000_000ns)
    verifyExpr("0.5day",  43_200_000_000_000ns)

    // uri
    verifyExpr("`x`",  `x`)
    verifyExpr("`http://fandev/path/file?query#frag`", `http://fandev/path/file?query#frag`)

    // type
    verifyExpr("Str.type", Str.type)
    verifyExpr("Str[].type", Str[].type)
    verifyExpr("Str:Int.type", Str:Int.type)
    verifyExpr("|Str a->Bool|.type", |Str a->Bool|.type)

    // range
    verifyExpr("2..3",  2..3)
    verifyExpr("2...3", 2...3)

    // list
    verifyExpr("[,]", [,])
    verifyExpr("Str[,]", Str[,])
    verifyExpr("Int[3]", Int[3])
    verifyExpr("[0]",  [0])
    verifyExpr("[0,1]", [0,1])
    verifyExpr("Obj[0,1]", Obj[0,1])
    verifyExpr("[2,2f]", Num[2,2f])

    // map
    verifyExpr("[:]", [:])
    verifyExpr("Int:Str[:]", Int:Str[:])
    verifyExpr("[2:2f]", [2:2f])
    verifyExpr("[2:2f, 3:3]", Int:Num[2:2f, 3:3])
    verifyExpr("[2:2f, 3f:3f]", Num:Float[2:2f, 3f:3f])
  }

//////////////////////////////////////////////////////////////////////////
// Locals
//////////////////////////////////////////////////////////////////////////

  Void testLocals()
  {
    verifyExpr("a", 3, 3)
    verifyExpr("b", 2, 1, 2)
    verifyExpr("c", 3, 1, 2, "c := 3;")
    verifyExpr("c", 3, 1, 2, "Int c := 3;")
    verifyExpr("c", 7, 1, 2, "Int c; c = 7;")
    verifyExpr("c", null, 1, 2, "Int c;")
  }

//////////////////////////////////////////////////////////////////////////
// Operators
//////////////////////////////////////////////////////////////////////////

  Void testOperators()
  {
    verifyExpr("!a", true, false)
    verifyExpr("!a", false, true)

    verifyExpr("+a", 2, 2)
    verifyExpr("+a", -2f, -2f)

    verifyExpr("a === b", true, 3, 3)
    verifyExpr("a !== b", false, 3, 3)

    verifyExpr("a == null",  false, 3)
    verifyExpr("a === null", false, 3)
    verifyExpr("a != null",  true,  3)
    verifyExpr("a !== null", true,  3)
    verifyExpr("null == a",  true,  null, null, "Str a := null;")
    verifyExpr("null === a", true,  null, null, "Str a := null;")
    verifyExpr("null != a",  false, null, null, "Str a := null;")
    verifyExpr("null !== a", false, null, null, "Str a := null;")

    verifyExpr("a || b", true, true, true)
    verifyExpr("a || b", true, false, true)
    verifyExpr("a || b", true, true, false)
    verifyExpr("a || b", false, false, false)

    verifyExpr("a && b", true, true, true)
    verifyExpr("a && b", false, false, true)
    verifyExpr("a && b", false, true, false)
    verifyExpr("a && b", false, false, false)

    verifyExpr("(Obj)a is Str", false, 4)
    verifyExpr("(Obj)a is Str", true, "x")

    verifyExpr("(Obj)a as Str", null, 4)
    verifyExpr("(Obj)a as Str", "x", "x")

    verifyExpr("(Str)a", "x", "x")
    verifyErr(CastErr.type) |,| { verifyExpr("(Str)((Obj)a)", null, 3) }

    verifyExpr("true ? a : b", 1, 1, 2)
    verifyExpr("false ? a : b", 2, 1, 2)
  }

//////////////////////////////////////////////////////////////////////////
// Call
//////////////////////////////////////////////////////////////////////////

  Void testCall()
  {
    // import static with arg
    verifyExpr("Str.spaces(1)", " ")
    verifyExpr("sys::Str.spaces(2)", "  ")

    // import static no args
    verifyExpr("Sys.homeDir()", Sys.homeDir)
    verifyExpr("sys::Sys.homeDir()", Sys.homeDir)
    verifyExpr("Sys.homeDir", Sys.homeDir)
    verifyExpr("sys::Sys.homeDir", Sys.homeDir)

    // import instance target
    verifyExpr("3.increment()", 4)
    verifyExpr("5.increment", 6)
    verifyExpr("2.increment().increment.isEven", true)

    // default args
    verifyExpr("\"abcb\".index(\"b\")", 1)
    verifyExpr("\"abcb\".index(\"b\", 2)", 3)

    // instance myself
    verifyExpr("ifoo", "ifoo")
    verifyExpr("ifoo()", "ifoo")
    verifyExpr("this.ifoo", "ifoo")
    verifyExpr("this.ifoo()", "ifoo")

    // static myself
    verifyExpr("sfoo", "sfoo")
    verifyExpr("sfoo()", "sfoo")
    verifyExpr("Foo.sfoo", "sfoo")
    verifyExpr("Foo.sfoo()", "sfoo")
    verifyExpr("${podName}::Foo.sfoo", "sfoo")
    verifyExpr("${podName}::Foo.sfoo()", "sfoo")

    // generics
    verifyExpr("x.negate", -20, [0, 10, 20, 30], null, "Int x; x = a[2];")
  }

//////////////////////////////////////////////////////////////////////////
// Dynamic Call
//////////////////////////////////////////////////////////////////////////

  Void testDynamicCall()
  {
    // dynamic calls
    verifyExpr("((Obj)3)->negate", -3)
    verifyExpr("((Obj)3)->plus(2)", 5)
    verifyExpr("((Obj)3)->plus = 6", 9)
  }

//////////////////////////////////////////////////////////////////////////
// Shortcuts
//////////////////////////////////////////////////////////////////////////

  Void testShortcuts()
  {
    // math operators
    verifyExpr("-a", -7, 7)
    verifyExpr("a + b", 3, 1, 2)
    verifyExpr("a - b", 2, 5, 3)
    verifyExpr("a * b", 12, 4, 3)
    verifyExpr("a / b", 3, 12, 4)
    verifyExpr("a % b", 1, 5, 2)

    // bitwise operators
    verifyExpr("~a", 0xffff_ffff_ffff_5432, 0xabcd)
    verifyExpr("a >> b", 0xa, 0xab, 4)
    verifyExpr("a << b", 0xab0, 0xab, 4)
    verifyExpr("a & b", 0x8a, 0xab, 0x9e)
    verifyExpr("a | b", 0xbf, 0xab, 0x9e)
    verifyExpr("a ^ b", 0x35, 0xab, 0x9e)

    // equality
    verifyExpr("a == b", true, "x", "x")
    verifyExpr("a == null", false, "x", "x")
    verifyExpr("null == a", false, "x", "x")
    verifyExpr("a != b", false, "x", "x")
    verifyExpr("a != null", true, "x", "x")
    verifyExpr("null != a", true, "x", "x")

    // comparisons
    verifyExpr("a < b", true, 2, 3)
    verifyExpr("a < b", false, 2, 2)
    verifyExpr("a <= b", true, 2, 3)
    verifyExpr("a <= b", true, 2, 2)
    verifyExpr("a <= b", false, 2, 1)
    verifyExpr("a > b", true, 4, 3)
    verifyExpr("a > b", false, 4, 9)
    verifyExpr("a >= b", true, 4, 3)
    verifyExpr("a >= b", true, 2, 2)
    verifyExpr("a >= b", false, 2, 4)
    verifyExpr("a <=> b", 0, 3, 3)
    verifyExpr("a <=> b", -1, 3, 7)
    verifyExpr("a <=> b", +1, -1, -2)

    // get
    verifyExpr("a[b]", 'b', "abc", 1)
    verifyExpr("a[b]", 'c', "abc", -1)

    // set
    verifyExpr("a[b] = 99", [0, 99, 2], [0, 1, 2], 1)
    verifyExpr("a[b] = 99", [99, 1, 2], [0, 1, 2], -3)

    // slice
    verifyExpr("a[b]", [1,2], [0, 1, 2, 3], 1..2)
    verifyExpr("a[b]", [1], [0, 1, 2, 3], 1...2)
    verifyExpr("a[b]", [2, 3], [0, 1, 2, 3], -2..-1)
    verifyExpr("a[b]", [2], [0, 1, 2, 3], -2...-1)

    // TODO: splice
  }

//////////////////////////////////////////////////////////////////////////
// Assignments
//////////////////////////////////////////////////////////////////////////

  Void testAssignments()
  {
    verifyAssignments("a")
  }

  Void verifyAssignments(Str v)
  {
    verifyExpr("$v", 2, 1, 2, "$v = b;")

    verifyExpr("$v", 5, 2, 3, "$v = a; $v+= b;")
    verifyExpr("$v", -1, 2, 3, "$v = a; $v-= b;")
    verifyExpr("$v", 6, 2, 3, "$v = a; $v*= b;")
    verifyExpr("$v", 3, 6, 2, "$v = a; $v/= b;")
    verifyExpr("$v", 2, 8, 3, "$v = a; $v%= b;")

    verifyExpr("$v", 0xab, 0xabcd, 8, "$v = a; $v>>= b;")
    verifyExpr("$v", 0xabcd0, 0xabcd, 4, "$v = a; $v<<= b;")
    verifyExpr("$v", 0x8a, 0xab, 0x9e, "$v = a; $v&= b;")
    verifyExpr("$v", 0xbf, 0xab, 0x9e, "$v = a; $v|= b;")
    verifyExpr("$v", 0x35, 0xab, 0x9e, "$v = a; $v^= b;")
  }

//////////////////////////////////////////////////////////////////////////
// Increment Operators
//////////////////////////////////////////////////////////////////////////

  Void testIncrementOps()
  {
    verifyIncrementOps("a", true)
  }

  Void verifyIncrementOps(Str v, Bool testFloat)
  {
    verifyExpr("++$v", 3, 2, 0, "$v = a;")
    verifyExpr("$v++", 2, 2, 0, "$v = a;")
    verifyExpr("[$v,b]", [2,3], 0, 2, "$v = b++;")
    verifyExpr("[$v,b]", [3,3], 0, 2, "$v = ++b;")

    verifyExpr("--$v", 1, 2, 0, "$v = a;")
    verifyExpr("$v--", 2, 2, 0, "$v = a;")
    verifyExpr("[$v,b]", [2,1], 0, 2, "$v = b--;")
    verifyExpr("[$v,b]", [1,1], 0, 2, "$v = --b;")

    verifyExpr("--a == b", true, 3, 2)

    if (testFloat)
    {
      verifyExpr("++a", 3f, 2f)
      verifyExpr("a++", 2f, 2f)
      verifyExpr("[a,b]", [2f,3f], 0f, 2f, "a = b++;")
      verifyExpr("[a,b]", [3f,3f], 0f, 2f, "a = ++b;")

      verifyExpr("--a", 1f, 2f)
      verifyExpr("a--", 2f, 2f)
      verifyExpr("[a,b]", [2f,1f], 0f, 2f, "a = b--;")
      verifyExpr("[a,b]", [1f,1f], 0f, 2f, "a = --b;")
    }
  }

  Void testIncrementMore()
  {
    src := "
      class Foo
      {
        Int f() { return a += b++ }
        Int g() { return a += ++b }
        Void h() { 3.times |,| { a = (b++) } }
        Int i() { return a += b++ + (c++).toInt }
        Void j() { x := 2; a = |->Int| { return x++ }.call0; b = x } // cvar field

        Int a := 2
        Int b := 3
        Float c := 4.0
      }"
     compile(src)

     t := pod.types.first

     o := t.make
     verifyEq(t.method("f").callOn(o, null), 5)
     verifyEq(o->a, 5)
     verifyEq(o->b, 4)

     o = t.make
     verifyEq(t.method("g").callOn(o, null), 6)
     verifyEq(o->a, 6)
     verifyEq(o->b, 4)

     o = t.make
     verifyEq(t.method("h").callOn(o, null), null)
     verifyEq(o->a, 5)
     verifyEq(o->b, 6)

     o = t.make
     verifyEq(t.method("i").callOn(o, null), 9)
     verifyEq(o->a, 9)
     verifyEq(o->b, 4)
     verifyEq(o->c, 5.0)

     o = t.make
     verifyEq(t.method("j").callOn(o, null), null)
     verifyEq(o->a, 2)
     verifyEq(o->b, 3)
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  Void testFields()
  {
    // don't rely on field initializers in this test
    verifyExpr("f", 7, 0, 0, "f = 7;")
    verifyAssignments("f")
    verifyIncrementOps("f", false)

    // const static
    verifyExpr("sf", 1972)

    // generics
    verifyExpr("tue", Weekday.tue, null, null, "Weekday tue := Weekday.values[2];")
  }

//////////////////////////////////////////////////////////////////////////
// Str Concat
//////////////////////////////////////////////////////////////////////////

  Void testStrConcat()
  {
    verifyExpr("\"\$a \$b\"", "4 5", 4, 5)
    verifyExpr("\"x\" + a", "x7", 7)
    verifyExpr("a + \"x\"", "7x", 7)

    verifyExpr("a += \"y\"", "xy", "x")
  }

//////////////////////////////////////////////////////////////////////////
// Test Simple Literal
//////////////////////////////////////////////////////////////////////////

  Void testSimples()
  {
    verifyExpr("Version(\"3.4.9\")", Version.make([3,4,9]))
    verifyExpr("Version(\"\${a}.99\")", Version.make([3,6,99]), "3.6")

    // errors
    verifyErrors("class Foo
      {
        static Obj a() { return SA(\"x\") }
        static Obj b() { return SB(\"x\") }
        static Obj c() { return SC(\"x\") }
        static Obj d() { return SD(\"x\") }
        static Obj e() { return Version(5) }
      }

      class SA {}
      class SB { static Void fromStr() {} }
      class SC { static SC fromStr(Int x) {return null} }
      class SD { SD fromStr(Str s, Int x) {return null} }
      ",
      [ 3, 33, "Simple type '$podName::SA' missing 'fromStr' method",
        4, 33, "Simple type '$podName::SB.fromStr' returns wrong type",
        4, 33, "Simple type '$podName::SB.fromStr' not enough parameters",
        5, 33, "Simple type '$podName::SC.fromStr' first parameter not Str",
        6, 33, "Simple type '$podName::SD.fromStr' not static method",
        6, 33, "Simple type '$podName::SD.fromStr' too many parameters",
        7, 41, "Simple literal requires 'Str' argument, not 'sys::Int'",
       ])
  }

//////////////////////////////////////////////////////////////////////////
// Test With Blocks
//////////////////////////////////////////////////////////////////////////

  Void testWithBlocks()
  {
    compile("
      class Acme
      {
        static Obj a() { return Foo.make { i = 77 } }
        static Obj b() { return Foo.make { i = 9; j += 6 } }
        static Obj c() { return Foo.make { inc } }
        static Obj d() { return Foo { i = 10; j = 11; inc } }
        static Obj e() { return Str[,] { size = 33 } }
        static Obj f()
        {
          return Foo
          {
            i=10;
            kid = Foo
            {
              i=20;
              kid = Foo {i=30}
              kid.j = 300
            }
          }
        }

        Foo x := Foo { i=-3; j=-5 }
      }


      class Foo
      {
        Foo inc() { i++; j++; return this }
        Int i := 2
        Int j := 5
        Foo kid
      }
      ")

     t := pod.types.first

     x := t.method("a").call0
     verifyEq(x->i, 77)
     verifyEq(x->j, 5)

     x = t.method("b").call0
     verifyEq(x->i, 9)
     verifyEq(x->j, 11)

     x = t.method("c").call0
     verifyEq(x->i, 3)
     verifyEq(x->j, 6)

     x = t.method("d").call0
     verifyEq(x->i, 11)
     verifyEq(x->j, 12)

     x = t.method("e").call0
     verifyEq(x->type, Str[].type)
     verifyEq(x->size, 33)
     verifyEq(x->capacity, 33)

     x = t.method("f").call0
     verifyEq(x->i, 10)
     verifyEq(x->kid->i, 20)
     verifyEq(x->kid->kid->i, 30)
     verifyEq(x->kid->kid->j, 300)

     x = t.field("x").get(t.make)
     verifyEq(x->i, -3)
     verifyEq(x->j, -5)

    // errors
    verifyErrors("class Foo
      {
        static Obj a() { return A {} }
        static Obj b() { return B { x = 4 } }
      }

      class A { new mk() {} }
      class B { }
      ",
      [ 3, 33, "Unknown slot '$podName::A.make'",
        4, 37, "Unknown slot '$podName::B.x'",
      ])

    // errors
    verifyErrors("class Foo
      {
        static Obj a() { return A { x } }
      }

      class A { Int x; Int y}
      ",
      [ 3, 37, "Not a statement",
      ])
  }

//////////////////////////////////////////////////////////////////////////
// Call Operator
//////////////////////////////////////////////////////////////////////////

  Void testCallOperator()
  {
    compile("
        class Foo
        {
          Func funcField := Foo.type.method(\"m4\").func

          static Int nine() { return 9 }
          static Func nineFunc() {  return Foo.type.method(\"nine\").func }

          static Int m1(Int a) { return a }
          static Int m4(Int a, Int b, Int c, Int d) { return d }
          static Int m8(Int a, Int b, Int c, Int d, Int e, Int f, Int g, Int h) { return h }

          Int i1(Int a) { return a }
          Int i4(Int a, Int b, Int c, Int d) { return d }
          Int i7(Int a, Int b, Int c, Int d, Int e, Int f, Int g) { return g }

          static Int callClosure(|->Int| c) { return c() }

          static Obj a()
          {
            m := Foo.type.method(\"nine\").func
            return m()
          }

          static Obj b()
          {
            m := |->Int| { return 69 }
            return m()
          }

          static Obj c(Int a, Int b)
          {
            m := |Int x, Int y->Int| { return x + y }
            return m(a, b)
          }

          static Obj d() { return Foo.type.method(\"nine\").func()() }

          static Obj e() { return ((Func)Foo.type.method(\"nineFunc\").func()())() }

          static Int f()
          {
            m := (|-> |->Int| |)Foo.type.method(\"nineFunc\").func
            return m()()
          }

          static Obj g() { return Foo.type.method(\"m1\").func()(7) }
          static Obj h() { return Foo.type.method(\"m4\").func()(10, 11, 12, 13) }
          static Obj i() { return Foo.type.method(\"m8\").func()(1, 2, 3, 4, 5, 6, 7, 8) }

          Obj j() { return Foo.type.method(\"i1\").func()(this, 6) }
          Obj k() { return Foo.type.method(\"i4\").func()(this, 101, 111, 121, 131) }
          Obj l() { return Foo.type.method(\"i7\").func()(this, -1, -2, -3, -4, -5, -6, -7) }

          Int m(Int p)
          {
            list := [ (|Int a->Int|) type.method(\"m1\").func() ]
            return list[0](p)
          }

          Obj o(Int p)
          {
            return (funcField)(0, 1, 2, p)
          }

          Obj q(Int p)
          {
            return Foo.type.method(\"callClosure\").func()() |->Int| { return p }
          }
        }
      ")

    // compiler.fpod.dump
    t := pod.types[0]
    obj := t.make
    verifyEq(obj->a, 9)
    verifyEq(obj->b, 69)
    verifyEq(obj->c(10, 3), 13)
    verifyEq(obj->d, 9)
    verifyEq(obj->e, 9)
    verifyEq(obj->f, 9)
    verifyEq(obj->g, 7)
    verifyEq(obj->h, 13)
    verifyEq(obj->i, 8)
    verifyEq(obj->j, 6)
    verifyEq(obj->k, 131)
    verifyEq(obj->l, -7)
    verifyEq(obj->m(54), 54)
    verifyEq(obj->o(33), 33)
    verifyEq(obj->q('x'), 'x')
  }

  Void testCallOperatorErrors()
  {
    verifyErrors("class Foo
      {
        static Void a(Str x) { x() }
        static Void b() { Str x; x() }
        static Void c() { x := 44; x() }
        static Void d()
        {
          m := |Int a, Int b, Int c, Int d, Int e, Int f, Int g, Int h, Int j| {}
          m(0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
        }

        static Void m9(Int a, Int b, Int c, Int d, Int e, Int f, Int g, Int h, Int j) {}
      }",
      [
        3, 32, "Cannot call local variable 'x' like a function",
        4, 34, "Cannot call local variable 'x' like a function",
        5, 36, "Cannot call local variable 'x' like a function",
        9, 11, "Tough luck - cannot use () operator with more than 8 arguments, use call(List)",
      ])
  }

//////////////////////////////////////////////////////////////////////////
// Errors
//////////////////////////////////////////////////////////////////////////

  Void testErrors()
  {
    // errors
    verifyErrors("class Foo
      {
        static Obj a() { return foobar }
        static Obj b() { return 3.foobar }
        static Obj c() { return 3.noway() }
        static Obj d() { return 3.nope(3) }
        static Obj e() { return sys::Str.foo }
        static Obj f() { return sys::Str.foo() }
        static Obj g(Int x) { x := 3 }
        static Obj h(Int y) { Int y; }
        static Obj i() { z := 3; z := 5 }
        static Obj j() { return foobar.x }
        static Obj k() { return 8f.foobar().x.y }
        static Obj l() { return foo + bar }
        static Obj m() { return (4.foo.ouch + bar().boo).rightOn }
        static Obj n(Str x) { return x++ }
        static Obj o(Str x) { return --x }
        static Obj q(Str x) { return x / 3 }
        static Obj r(Str x) { return x |= 3 }
      }",
      [ 3, 33, "Unknown variable 'foobar'",
        4, 35, "Unknown slot 'sys::Int.foobar'",
        5, 35, "Unknown method 'sys::Int.noway'",
        6, 35, "Unknown method 'sys::Int.nope'",
        7, 42, "Unknown slot 'sys::Str.foo'",
        8, 42, "Unknown method 'sys::Str.foo'",
        9, 31, "Variable 'x' is already defined in current block",
       10, 31, "Variable 'y' is already defined in current block",
       11, 34, "Variable 'z' is already defined in current block",
       12, 33, "Unknown variable 'foobar'",
       13, 36, "Unknown method 'sys::Float.foobar'",
       14, 33, "Unknown variable 'foo'",
       14, 39, "Unknown variable 'bar'",
       15, 36, "Unknown slot 'sys::Int.foo'",
       15, 47, "Unknown method '$podName::Foo.bar'",
       16, 38, "Unknown method 'sys::Str.increment'",
       17, 38, "Unknown method 'sys::Str.decrement'",
       18, 38, "Unknown method 'sys::Str.slash'",
       19, 38, "Unknown method 'sys::Str.pipe'",
       ])
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  Void verifyExpr(Str code, Obj result, Obj a := null, Obj b := null, Str more := "")
  {
    params := ""
    if (a != null) params = a.type.signature+ " a"
    if (b != null) params += ", " + b.type.signature + " b"

    src := "
      class Foo
      {
        new make() { return }

        Str ifoo() { return \"ifoo\" }
        static Str sfoo() { return \"sfoo\" }

        Obj func($params) { $more return $code }

        Int f
        const static Int sf := 1972
      }"
     compile(src)

     t := pod.types.first
     instance := t.method("make").call0
     actual := t.method("func").call([instance, a, b])
     verifyEq(actual, result)
   }

}