//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   23 Mar 08  Brian Frank  Creation
//

**
** FuncTest
**
class FuncTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Curry Calls
//////////////////////////////////////////////////////////////////////////

  Void testCurryCalls()
  {
    // verify binding/calling
    verifyCurry |->Str| { return "" }
    verifyCurry |Str a->Str| { return a }
    verifyCurry |Str a, Str b->Str| { return a + b  }
    verifyCurry |Str a, Str b, Str c->Str| { return a + b + c }
    verifyCurry |Str a, Str b, Str c, Str d->Str| { return a + b + c + d }
    verifyCurry |Str a, Str b, Str c, Str d, Str e->Str| { return a + b + c + d + e }
    verifyCurry |Str a, Str b, Str c, Str d, Str e, Str f->Str| { return a + b + c + d + e + f }
    verifyCurry |Str a, Str b, Str c, Str d, Str e, Str f, Str g->Str| { return a + b + c + d + e + f + g }
    verifyCurry |Str a, Str b, Str c, Str d, Str e, Str f, Str g, Str h->Str| { return a + b + c + d + e + f + g + h }
    verifyCurry |Str a, Str b, Str c, Str d, Str e, Str f, Str g, Str h, Str i->Str| { return a + b + c + d + e + f + g + h + i }
    verifyCurry |Str a, Str b, Str c, Str d, Str e, Str f, Str g, Str h, Str i, Str j->Str| { return a + b + c + d + e + f + g + h + i + j }
  }

  Void verifyCurry(Func f)
  {
    args := Str[,]
    expected := ""
    f.params.size.times |Int i|
    {
      ch := ('a' + i).toChar
      args.add(ch)
      expected += ch
    }

    verifyEq(f.call(args), expected)

    args.size.times |Int i|
    {
      g := f.curry(args[0...i])
      if (i == 0) verifySame(f, g)

      // call(List)
      a := args[i..-1]
      verifyEq(g.call(a), expected)

      // callX
      switch (a.size)
      {
        case 0: verifyEq(g.call0(), expected)
        case 1: verifyEq(g.call1(a[0]), expected)
        case 2: verifyEq(g.call2(a[0], a[1]), expected)
        case 3: verifyEq(g.call3(a[0], a[1], a[2]), expected)
        case 4: verifyEq(g.call4(a[0], a[1], a[2], a[3]), expected)
        case 5: verifyEq(g.call5(a[0], a[1], a[2], a[3], a[4]), expected)
        case 6: verifyEq(g.call6(a[0], a[1], a[2], a[3], a[4], a[5]), expected)
        case 7: verifyEq(g.call7(a[0], a[1], a[2], a[3], a[4], a[5], a[6]), expected)
        case 8: verifyEq(g.call8(a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]), expected)
      }

      // callOn
      if (a.size >= 1) verifyEq(g.callOn(a[0], a[1..-1]), expected)

      // curry operator
      Func c := null
      switch (i)
      {
        case 0: c = f
        case 1: c = &f(args[0])
        case 2: c = &f(args[0], args[1])
        case 3: c = &f(args[0], args[1], args[2])
        case 4: c = &f(args[0], args[1], args[2], args[3])
        case 5: c = &f(args[0], args[1], args[2], args[3], args[4])
        case 6: c = &f(args[0], args[1], args[2], args[3], args[4], args[5])
        case 7: c = &f(args[0], args[1], args[2], args[3], args[4], args[5], args[6])
        case 8: c = &f(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7])
      }
      if (c != null) verifyEq(c.call(a), expected)

    }

    x := f.curry(args)
    verifyEq(x.call([,]), expected)
    verifyEq(x.call(["x", "y"]), expected)
    verifyEq(x.call0, expected)
    verifyEq(x.call1("x"), expected)
    verifyEq(x.call2("x", "y"), expected)
  }

//////////////////////////////////////////////////////////////////////////
// Curry Signatures
//////////////////////////////////////////////////////////////////////////

  Void testCurrySig()
  {
    Func f := |Bool b, Int i, Float f, Str s->Str| { return "$b $i $f $s" }
    verifyEq(f.type.signature, "|sys::Bool,sys::Int,sys::Float,sys::Str->sys::Str|")
    verifyEq(f.params[0].of, Bool.type)
    verifyEq(f.params[1].of, Int.type)
    verifyEq(f.params[2].of, Float.type)
    verifyEq(f.params[3].of, Str.type)
    verifyEq(f.returns, Str.type)

    g := f.curry([true])
    verifyEq(g.type.signature, "|sys::Int,sys::Float,sys::Str->sys::Str|")
    verifyEq(g.params[0].of, Int.type)
    verifyEq(g.params[1].of, Float.type)
    verifyEq(g.params[2].of, Str.type)
    verifyEq(g.returns, Str.type)

    h := f.curry([true, 9, 4f])
    verifyEq(h.type.signature, "|sys::Str->sys::Str|")
    verifyEq(h.params[0].of, Str.type)
    verifyEq(h.returns, Str.type)

    i := g.curry([7])
    verifyEq(i.type.signature, "|sys::Float,sys::Str->sys::Str|")
    verifyEq(i.params[0].of, Float.type)
    verifyEq(i.params[1].of, Str.type)
    verifyEq(i.returns, Str.type)

    verifyEq(f.call4(false, 8, 3f, "x"), "false 8 3.0 x")
    verifyEq(g.call3(33, 6f, "y"), "true 33 6.0 y")
    verifyEq(i.call([2f, "q"]), "true 7 2.0 q")
    verifyEq(i.call2(2f, "q"), "true 7 2.0 q")
    verifyEq(i.curry([2f, "q"]).call0, "true 7 2.0 q")
    verifyEq(i.curry([2f, "q"]).call1('x'), "true 7 2.0 q")

    verifyErr(ArgErr.type) |,| { f.curry([true, 8, 8f, "x", "y"]) }
    verifyErr(ArgErr.type) |,| { i.curry([8f, "x", null]) }
  }

//////////////////////////////////////////////////////////////////////////
// Curry Operator
//////////////////////////////////////////////////////////////////////////

  Void testCurryOp()
  {
    f := |Int a, Str b, Float c->Str| { return "$a $b $c" }
    verifyEq(f(1, "x", 2.0), "1 x 2.0")

    g := &f(1)
    verifyEq(g("x", 2.0), "1 x 2.0")

    h := &g("y")
    verifyEq(h(3.0), "1 y 3.0")

    i := &h(4.0)
    verifyEq(i(), "1 y 4.0")

    j := Int.type.method("minus").func
    verifyEq(j(3, 4), -1)

    k := &j(10)
    verifyEq(k(4), 6)

    l := &j(10, 8)
    verifyEq(l.call0, 2)

    m := &Str.type.method("spaces").func()(3)
    verifyEq(m(), "   ")

    verifySame(j.method, Int.type.method("minus"))
    verifySame((&Str.spaces).method, Str.type.method("spaces"))
    verifySame((&10.plus).method, Int.type.method("plus"))
  }

}