//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   20 Jun 06  Brian Frank  Creation
//

**
** ParamTest
**
class TryTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Catch All
//////////////////////////////////////////////////////////////////////////

  Void testCatchAll()
  {
    s := "start"
    try
    {
      s += " before "
      throw Err.make
      s += " wrong "
    }
    catch
    {
      s += "caught"
    }
    verifyEq(s, "start before caught");
  }

//////////////////////////////////////////////////////////////////////////
// Catch Same Type
//////////////////////////////////////////////////////////////////////////

  Void testCatchSameType()
  {
    s := "start"
    try
    {
      s += " before "
      throw IndexErr.make
      s += " wrong "
    }
    catch (IndexErr err)
    {
      s += "caught($err.type.name)"
    }
    verifyEq(s, "start before caught(IndexErr)");
  }

//////////////////////////////////////////////////////////////////////////
// Catch Super Type
//////////////////////////////////////////////////////////////////////////

  Void testCatchSuperType()
  {
    s := "start"
    try
    {
      s += " before "
      throw CatchTestIOErr.make("foo")
      s += " wrong "
    }
    catch (IOErr err)
    {
      s += "caught($err.type.name)"
    }
    verifyEq(s, "start before caught(CatchTestIOErr)");
  }

//////////////////////////////////////////////////////////////////////////
// Catch Wrong Type
//////////////////////////////////////////////////////////////////////////

  Void testCatchWrongType()
  {
    verifyErr(IndexErr.type) |,|
    {
      try
      {
        throw IndexErr.make("foo")
      }
      catch (IOErr err)
      {
        fail()
      }
    }
  }

//////////////////////////////////////////////////////////////////////////
// Catch Multiple
//////////////////////////////////////////////////////////////////////////

  Void testCatchMultiple()
  {
    verifyEq(catchMultiple(IOErr.make), IOErr.type)
    verifyEq(catchMultiple(IndexErr.make), IndexErr.type)
    verifyEq(catchMultiple(ArgErr.make), ArgErr.type)
    verifyErr(ReadonlyErr.type) |,| { catchMultiple(ReadonlyErr.make) }
  }

  Type catchMultiple(Err err)
  {
    try
    {
      throw err
    }
    catch (IOErr e)
    {
      return IOErr.type
    }
    catch (IndexErr e)
    {
      return IndexErr.type
    }
    catch (ArgErr e)
    {
      return ArgErr.type
    }
  }

//////////////////////////////////////////////////////////////////////////
// Catch Nested
//////////////////////////////////////////////////////////////////////////

  Void testNested()
  {
    s := "";
    try
    {
      s += "a";
      try
      {
        s += " b";
        throw IndexErr.make
        s += " c";
      }
      catch (IndexErr e)
      {
        s += " IndexErr";
      }
      s += " d"
    }
    catch (NullErr e)
    {
      s += " NullErr";
    }

    verifyEq(s, "a b IndexErr d");

    s = "";
    try
    {
      s += "a";
      try
      {
        s += " b";
        throw NullErr.make
        s += " c";
      }
      catch (IndexErr e)
      {
        s += " IndexErr";
      }
      s += " d"
    }
    catch (NullErr e)
    {
      s += " NullErr";
    }

    verifyEq(s, "a b NullErr");
    s = "";
    try
    {
      s += "a";
      try
      {
        s += " b";
        throw NullErr.make
        s += " c";
      }
      catch (NullErr e)
      {
        s += " NullErrA";
      }
      s += " d"
    }
    catch (NullErr e)
    {
      s += " NullErrB";
    }
    verifyEq(s, "a b NullErrA d");
  }

//////////////////////////////////////////////////////////////////////////
// Runtime Mapping Test
//////////////////////////////////////////////////////////////////////////

  **
  ** The goal here is to cause exceptions which are raised by the
  ** runtime rather than our system code - we want to ensure that
  ** they are mapped to their Fan counterparts correctly.  We try
  ** one in three scenerios: Test.verifyErr, catch, and reflection
  **
  Void testRuntimeMapping()
  {
    try { throwCastErr } catch (CastErr e) { } catch { fail }
    verifyErr(CastErr.type) |,| { throwCastErr }
    verifyErr(CastErr.type) |,| { TryTest.type.method("throwCastErr").call0 }

    try { throwNullErr } catch (NullErr e) { } catch { fail }
    verifyErr(NullErr.type) |,| { throwNullErr }
    verifyErr(NullErr.type) |,| { TryTest.type.method("throwNullErr").call0 }

    try { throwIndexErr } catch (IndexErr e) { } catch { fail }
    verifyErr(IndexErr.type) |,| { throwIndexErr }
    verifyErr(IndexErr.type) |,| { TryTest.type.method("throwIndexErr").call0 }

    try { throwIOErr } catch (IOErr e) { } catch { fail }
    verifyErr(IOErr.type) |,| { throwIOErr }
    verifyErr(IOErr.type) |,| { TryTest.type.method("throwIOErr").call0 }

    try { throwInterruptedErr } catch (InterruptedErr e) { } catch { fail }
    verifyErr(InterruptedErr.type) |,| { throwInterruptedErr }
    verifyErr(InterruptedErr.type) |,| { TryTest.type.method("throwInterruptedErr").call0 }
  }

  static Str throwCastErr() { Obj four := 4; return (Str)four }
  static Void throwNullErr() { Str a := null; a.size }
  static Void throwIndexErr() { x := "foo"[100] }
  static Void throwIOErr() { throw IOErr.make /* not really a good test */ }
  static Void throwInterruptedErr() { throw InterruptedErr.make /* not really a good test */ }
}

//////////////////////////////////////////////////////////////////////////
// CatchTestIOErr
//////////////////////////////////////////////////////////////////////////

const class CatchTestIOErr : IOErr
{
  new make(Str msg, Err cause := null) : super(msg, cause) {}
}