//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   24 Mar 06  Brian Frank  Creation
//

**
** TypeTest
**
@testSysByStr=["alpha", "beta"]
@testSysByType=[SerB.type, MxB.type]
class TypeTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Find
//////////////////////////////////////////////////////////////////////////

  Void testIdentity()
  {
    verifyEq(this.type.isImmutable, true)
    verifySame(this.type.toImmutable, this.type)
  }

//////////////////////////////////////////////////////////////////////////
// Find
//////////////////////////////////////////////////////////////////////////

  Void testFind()
  {
    verifySame(Type.find("sys::Int"), Int.type)
    verifySame(Type.find("sys::Str[]"), Str[].type)
    verifySame(Type.find("sys::notHereFoo", false), null)
    verifyErr(UnknownTypeErr.type) |,| { Type.find("sys::notHereFoo") }
    verifyErr(UnknownPodErr.type) |,| { Type.find("notHereFoo::Duh") }
    verifyErr(ArgErr.type) |,| { Type.find("sys") }
    verifyErr(ArgErr.type) |,| { Type.find("sys::") }
    verifyErr(ArgErr.type) |,| { Type.find("::sys") }
    verifyErr(ArgErr.type) |,| { Type.find("[]") }
  }

//////////////////////////////////////////////////////////////////////////
// TypeDb
//////////////////////////////////////////////////////////////////////////

  Void testFindByFacet()
  {
    verifyErr(Err.type) |,| { Type.findByFacet("testSysKeyFoo", "") }

    x := Type.findByFacet("testSysByStr", "zeta")
    verifyEq(x.size, 0)
    verifyEq(x.isRO, true)

    x = Type.findByFacet("testSysByStr", "alpha")
    verifyEq(x.size, 2)
    verifyEq(x.isRO, true)
    verify(x.contains(TypeTest.type))
    verify(x.contains(FacetsTest.type))

    x = Type.findByFacet("testSysByStr", "beta")
    verifyEq(x, [TypeTest.type])

    x = Type.findByFacet("testSysByType", Str.type)
    verifyEq(x.size, 0)
    verifyEq(x.isRO, true)

    x = Type.findByFacet("testSysByType", SerA.type)
    verifyEq(x, [FacetsTest.type])

    x = Type.findByFacet("testSysByType", SerB.type)
    verifyEq(x, [TypeTest.type])

    x = Type.findByFacet("testSysByType", SerB.type, false)
    verifyEq(x, [TypeTest.type])

    x = Type.findByFacet("testSysByType", SerB.type, true)
    verifyEq(x.isRO, true)
    verifyEq(x, [TypeTest.type, FacetsTest.type])

    verifyEq(Type.findByFacet("testSysByType", MxB.type), [TypeTest.type])
    verifyEq(Type.findByFacet("testSysByType", MxAB.type), Type[,])
    verifyEq(Type.findByFacet("testSysByType", MxAB.type, true), [TypeTest.type])
    verifyEq(Type.findByFacet("testSysByType", MxClsAB.type, true), [TypeTest.type])
  }

//////////////////////////////////////////////////////////////////////////
// Flags
//////////////////////////////////////////////////////////////////////////

  Void testFlags()
  {
    // isAbstract
    verifyEq(Test.type.isAbstract, true)
    verifyEq(type.isAbstract, false)

    // isClass
    verifyEq(type.isClass, true)
    verifyEq(EnumAbc.type.isClass, false)
    verifyEq(MxA.type.isClass, false)

    // isEnum
    verifyEq(type.isEnum, false)
    verifyEq(EnumAbc.type.isEnum, true)
    verifyEq(MxA.type.isEnum, false)

    // isFinal
    verifyEq(Bool.type.isFinal, true)
    verifyEq(Test.type.isFinal, false)

    // isInternal
    verifyEq(type.isInternal, false)
    verifyEq(EnumAbc.type.isInternal, true)

    // isMixin
    verifyEq(type.isMixin, false)
    verifyEq(EnumAbc.type.isMixin, false)
    verifyEq(MxA.type.isMixin, true)

    // isPublic
    verifyEq(type.isPublic, true)
    verifyEq(EnumAbc.type.isPublic, false)

    // isSynthetic
    // test below in testSynthetic()
  }

//////////////////////////////////////////////////////////////////////////
// Mixins
//////////////////////////////////////////////////////////////////////////

  Void testMixins()
  {
    verifyEq(Obj.type.mixins, Type[,])
    verifyEq(Obj.type.mixins.isRO, true)

    verifyEq(MxClsAB.type.mixins, [MxA.type, MxB.type])
    verifyEq(MxClsAB.type.mixins.isRO, true)
  }

//////////////////////////////////////////////////////////////////////////
// Inheritance
//////////////////////////////////////////////////////////////////////////

  Void testInheritance()
  {
    verifyEq(Obj.type.inheritance, [Obj.type])
    verifyEq(Num.type.inheritance, [Num.type, Obj.type])
    verifyEq(Int.type.inheritance, [Int.type, Num.type, Obj.type])

    // test Void which is a special case
    verifyEq(Void.type.inheritance, [Void.type])

    // mixin types tested in MixinTest.testType
  }

//////////////////////////////////////////////////////////////////////////
// Fits
//////////////////////////////////////////////////////////////////////////

  Void testFits()
  {
    verify(Float.type.fits(Float.type))
    verify(Float.type.fits(Num.type))
    verify(Float.type.fits(Obj.type))
    verifyFalse(Float.type.fits(Str.type))
    verifyFalse(Obj.type.fits(Float.type))

    // void doesn't fit anything
    verifyFalse(Void.type.fits(Obj.type))
    verifyFalse(Obj.type.fits(Void.type))

    // TODO: add mixin types
  }

//////////////////////////////////////////////////////////////////////////
// Generics
//////////////////////////////////////////////////////////////////////////

  Void testIsGeneric()
  {
    verifyEq(Obj.type.isGeneric,    false)
    verifyEq(Str.type.isGeneric,    false)
    verifyEq(Str[].type.isGeneric,  false)
    verifyEq(Method.type.isGeneric, false)
    verifyEq(List.type.isGeneric,   true)
    verifyEq(Map.type.isGeneric,    true)
    verifyEq(Func.type.isGeneric,   true)
  }

  Void testParams()
  {
    verifyEq(Str.type.params, Str:Type[:])
    verifyEq(Str.type.params.isRO, true)

    verifyEq(Str[].type.params, ["V":Str.type, "L":Str[].type])
    verifyEq(Str[].type.params.isRO, true)

    verifyEq(Int:Slot[].type.params, ["K":Int.type, "V":Slot[].type, "M":Int:Slot[].type])
    verifyEq(Int:Slot[].type.params.isRO, true)

    verifyEq(|Int a, Float b->Bool|.type.params, ["A":Int.type, "B":Float.type, "R":Bool.type])
    verifyEq(|Int a, Float b->Bool|.type.params.isRO, true)
  }

  Void testParameterization()
  {
    verifyEq(List.type.parameterize(["V":Bool.type]), Bool[].type)
    verifyEq(List.type.parameterize(["V":Bool[].type]), Bool[][].type)
    verifyErr(ArgErr.type) |,| { List.type.parameterize(["X":Bool[].type]) }

    verifyEq(Map.type.parameterize(["K":Str.type, "V":Slot.type]), Str:Slot.type)
    verifyEq(Map.type.parameterize(["K":Str.type, "V":Int[].type]), Str:Int[].type)
    verifyErr(ArgErr.type) |,| { Map.type.parameterize(["V":Bool[].type]) }
    verifyErr(ArgErr.type) |,| { Map.type.parameterize(["K":Bool[].type]) }

    verifyEq(Func.type.parameterize(["R":Void.type]), |,|.type)
    verifyEq(Func.type.parameterize(["A":Str.type, "R":Int.type]), |Str a->Int|.type)
    verifyEq(Func.type.parameterize(["A":Str.type, "B":Bool.type, "C":Int.type, "R":Float.type]),
      |Str a, Bool b, Int c->Float|.type)
    verifyErr(ArgErr.type) |,| { Func.type.parameterize(["A":Bool[].type]) }

    verifyErr(UnsupportedErr.type) |,| { Str.type.parameterize(["X":Void.type]) }
    verifyErr(UnsupportedErr.type) |,| { Str[].type.parameterize(["X":Void.type]) }
  }

  Void testToListOf()
  {
    verifyEq(Str.type.toListOf, Str[].type)
    verifyEq(Str[].type.toListOf, Str[][].type)
    verifyEq(Str[][].type.toListOf, Str[][][].type)
    verifyEq(Str:Buf.type.toListOf, [Str:Buf][].type)
  }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  Void testMake()
  {
    verify(File.type.make([`foo`]) is File)
    verifyEq(File.type.make([`foo`])->uri, `foo`)
    //verifyErr(Err.type) |,| { Bool.type.make }
  }

//////////////////////////////////////////////////////////////////////////
// Synthetic
//////////////////////////////////////////////////////////////////////////

  Void testSynthetic()
  {
    type.pod.types.each |Type t|
    {
      verifyEq(t.isSynthetic, t.name.index("\$") != null, t.toStr)
      verifySlotsSynthetic(t)
    }
  }

  Void verifySlotsSynthetic(Type t)
  {
    t.slots.each |Slot slot|
    {
      if (slot.parent.isSynthetic || slot.name.index("\$") != null)
        verify(slot.isSynthetic, slot.toStr)
    }
  }

//////////////////////////////////////////////////////////////////////////
// Generic Parameters
//////////////////////////////////////////////////////////////////////////

  Void testGenericParameters()
  {
    // TODO - what do we really want for these guys?
    v := List.type.method("get").returns
    verifyEq(v.name, "V")
    verifyEq(v.qname, "sys::V")
    verifySame(v.pod, Obj.type.pod)
    verifySame(v.base, Obj.type)
    verifyEq(v.mixins, Type[,])
    verifyEq(v.mixins.ro, Type[,])
  }

}